using System;

namespace RoboidControl {

    /// <summary>
    /// A temperature sensor
    /// </summary>
    public class TemperatureSensor : Thing {
        /*
        /// <summary>
        /// Create a temperature sensor without communication abilities
        /// </summary>
        /// <param name="invokeEvent">Invoke a OnNewThing event when the thing has been created</param>
        public TemperatureSensor(bool invokeEvent = true) : base(Type.TemperatureSensor, invokeEvent) { }
        /// <summary>
        /// Create a temperature sensor for a participant
        /// </summary>
        /// <param name="owner">The participant for with the sensor is needed</param>
        /// <param name="thingId">The ID of the thing</param>
        /// <param name="invokeEvent">Invoke a OnNewThing event when the thing has been created</param>
        public TemperatureSensor(Participant owner, byte thingId = 0, bool invokeEvent = true) : base(owner, Type.TemperatureSensor, thingId, invokeEvent) { }
        */
        /// <summary>
        /// Create a new child temperature sensor
        /// </summary>
        /// <param name="parent">The parent thing</param>
        /// <param name="thingId">The ID of the thing, leave out or set to zero to generate an ID</param>
        /// <param name="invokeEvent">Invoke a OnNewThing event when the thing has been created</param>
        public TemperatureSensor(Thing parent) : base(parent) {
            this.type = Type.TemperatureSensor;
        }

        /// <summary>
        /// The measured temperature
        /// </summary>
        public float temperature = 0;

        /// <summary>
        /// Function used to generate binary data for this temperature sensor
        /// </summary>
        /// <returns>A byte array with the binary data</returns>
        public override byte[] GenerateBinary() {
            byte[] bytes = new byte[2];
            byte ix = 0;
            LowLevelMessages.SendFloat16(bytes, ref ix, this.temperature);
            return bytes;
        }

        /// <summary>
        /// Function to process the temperature from the binary data
        /// </summary>
        /// <param name="bytes">The binary data to process</param>
        public override void ProcessBinary(byte[] bytes) {
            byte ix = 0;
            this.temperature = LowLevelMessages.ReceiveFloat16(bytes, ref ix);
        }
    }

}